/*
 * Copyright (c) 2014-2018, Sonos, Inc.
 *
 * SPDX-License-Identifier: GPL-2.0
 *
 * sonos_attr_serialize.c.inc: Sonos attribute serialization
 */

#include "sonos_attr.h"
#include "sonos_nasn_serialize.h"
#include "sonos_signature_common.h"

static int sonosAttributeSerializeValue(const SonosAttribute* a,
                                        uint8_t** buf_p,
                                        const uint8_t* end)
{
    uint8_t* buf = *buf_p;

    switch SONOS_ATTR_ID(a->attributeId) {
    default:
    case SONOS_ATTR_ID_PADDING:
        PUT_BUF(a->x.attributeValue, a->attributeValueLen);
        break;

    case SONOS_ATTR_ID_MESSAGE_DIGEST:
    {
        PUT_BUF(a->x.attributeValue, a->attributeValueLen);
        break;
    }

    case SONOS_ATTR_ID_CONTENT_TYPE:
    {
        SonosAttributeContentType ct;

        ct = a->x.ct;
        ct.contentType = SONOS_NASN_CPU_TO_BE32(ct.contentType);

        PUT_BUF(&ct, sizeof ct);
        break;
    }

    case SONOS_ATTR_ID_SRK_REVOKE:
    {
        PUT_BUF(a->x.attributeValue, a->attributeValueLen);
        break;
    }

    case SONOS_ATTR_ID_WHITELIST:
    {
        size_t wlEntriesLen;
        const SonosFirmwareWhitelistHeader *hdr = &a->x.wl.header;

        switch (hdr->whitelistType) {
            case SONOS_FWW_TYPE_CPUID:
                wlEntriesLen = hdr->numEntries * SONOS_FWW_LEN_CPUID;
                break;
            case SONOS_FWW_TYPE_SERIAL:
                wlEntriesLen = hdr->numEntries * SONOS_FWW_LEN_SERIAL;
                break;
            case SONOS_FWW_TYPE_CPUID16:
                wlEntriesLen = hdr->numEntries * SONOS_FWW_LEN_CPUID16;
                break;
            default:
                return 0;
        }

        PUT_BUF(hdr->magic, sizeof(hdr->magic));
        PUT_ENUM(hdr->whitelistType, sonosSignatureIsValidWhitelistType);
        PUT_INT(hdr->numEntries);
        PUT_BUF(a->x.wl.entriesPtr, wlEntriesLen);
        break;
    }

    case SONOS_ATTR_ID_AR_VERSION:
    {
        SonosAttributeArVersion arv;

        arv = a->x.arv;
        arv.arVersion = SONOS_NASN_CPU_TO_BE32(arv.arVersion);

        PUT_BUF(&arv, sizeof arv);
        break;
    }

    }

    *buf_p = buf;
    return 1;
}

int sonosAttributeSerialize(const SonosAttribute* a,
                            uint8_t** buf_p,
                            const uint8_t* end)
{
    uint8_t* buf = *buf_p;

    PUT_INT(a->attributeId);
    if (SONOS_ATTR_IS_CRITICAL(a->attributeId) &&
        !sonosIsKnownAttribute(a->attributeId)) {
        return 0;
    }

    PUT_INT(a->attributeValueLen);

    if (!sonosAttributeSerializeValue(a, &buf, end)) {
        return 0;
    }

    *buf_p = buf;
    return 1;
}
